import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;
import swarm.collections.List;
import swarm.collections.ListImpl;
import swarm.random.NormalDist;
import swarm.random.NormalDistImpl;
import swarm.random.UniformIntegerDist;
import swarm.random.UniformIntegerDistImpl;

public class Customer extends SwarmObjectImpl {

  public int customerId;
  public ScnModelSwarm modelSwarm;
  public ListImpl orderList;
  public ListImpl manuList;
  public Trust[] myTrustHistory;

  public int[] manuQuotePrice;
  public int[] manuQuoteTime;
  public double[] manuTrust;
  public double[] manuFact;
  public double[] manuChoice;
  public int max_index;

  public NormalDist normalAbilityDist;
  public NormalDist normalBenevolenceDist;
  public NormalDist normalIntegrityDist;

  public int mNum;

  public Customer(Zone aZone, ScnModelSwarm s, ListImpl oList, ListImpl mList, Object generator, int cId) {
      super(aZone);
      modelSwarm = s;
      customerId = cId;
      int cNum = modelSwarm.numberOfCustomers;
      for (int i = 0; i < cNum; i++){
          orderList = oList;
      }// end of for
      mNum = modelSwarm.numberOfManufacturers;
      manuList = mList;

      myTrustHistory = new Trust[mNum];
      for (int i = 0; i < mNum; i++){
        myTrustHistory[i] = new Trust(modelSwarm.interestRate);
      }// end of for
      normalAbilityDist = new NormalDistImpl(modelSwarm.getZone(), generator);
      normalBenevolenceDist = new NormalDistImpl(modelSwarm.getZone(), generator);
      normalIntegrityDist = new NormalDistImpl(modelSwarm.getZone(), generator);
  }// end of constructor

  // choice the best manufacturer begin
  public void choice(int quantity){
      manuQuotePrice = new int[mNum];
      manuQuoteTime = new int[mNum];
      manuTrust = new double[mNum];
      manuFact = new double[mNum];
      manuChoice = new double[mNum];
      max_index = 0;
      for (int i = 0; i < mNum; i++){
          manuQuotePrice[i] = ((Manufacturer)manuList.atOffset(i)).queryPrice(quantity);
          manuQuoteTime[i] = ((Manufacturer)manuList.atOffset(i)).queryTime(quantity);
          manuTrust[i] = (myTrustHistory[i].getAbility() + myTrustHistory[i].getBenevolence() + myTrustHistory[i].getIntegrity())/3;
          int transactionTime = myTrustHistory[i].getTransactionTime();
          double propensity = transactionTime/ modelSwarm.orderList[i].getCount();

          if (propensity <= 0.2)
            manuFact[i] = Globals.env.uniformDblRand.getDoubleWithMin$withMax(0,0.2);
          else if (propensity <= 0.4 && propensity > 0.2)
            manuFact[i] = Globals.env.uniformDblRand.getDoubleWithMin$withMax(0.2,0.4);
          else if (propensity > 0.4 && propensity <= 0.6)
            manuFact[i] = Globals.env.uniformDblRand.getDoubleWithMin$withMax(0.4,0.6);
          else if (propensity > 0.6 && propensity <= 0.8)
            manuFact[i] = Globals.env.uniformDblRand.getDoubleWithMin$withMax(0.6,0.8);
          else
            manuFact[i] = Globals.env.uniformDblRand.getDoubleWithMin$withMax(0.8, 1.0);
        manuChoice[i] = manuFact[i] * manuTrust[i] + (1 - manuFact[i]) * (manuQuotePrice[i] + manuQuoteTime[i])/2;
      }
      for (int i = 0; i < mNum; i++){
        if (manuChoice[i] > manuChoice[max_index])
            max_index = i;
      }
  }

  public void assignOrder(int orderId){
      Order temp = (Order)orderList.atOffset(orderId);
      temp.setManuId(this.max_index);
      int sId = ((Manufacturer)manuList.atOffset(this.max_index)).max_index;
      temp.setSupplierId(sId);
      temp.setOrderPrice(this.manuQuotePrice[this.max_index]);
      temp.setContractTime(this.max_index);
  }
  public void exec_Order(){
      //System.out.println("In Customer exec_Order(),orderList.getCount() =" + orderList.getCount() );
      for (int i = 0; i < orderList.getCount(); i++){
          Order temp = (Order)orderList.atOffset(i);
          while(temp.triggerTime == Globals.env.getCurrentTime()&& temp.triggered == false){
              this.choice(temp.orderQuantity);
              this.assignOrder(i);
              Manufacturer manu = (Manufacturer) manuList.atOffset(temp.manufactId);
              int sCTime = manu.getCheckTime(temp.supplierId);
              int sDTime = manu.getDeliveryTime(temp.supplierId);
              int sBTime = manu.getBargainTime(temp.supplierId);
              int cTime = this.getCheckTime(temp.manufactId) + sCTime;
              int dTime = this.getDeliveryTime(temp.manufactId) + sDTime;
              int bTime = this.getBargainTime(temp.manufactId) + sBTime;
              myTrustHistory[temp.manufactId].addAbility(cTime);
              myTrustHistory[temp.manufactId].addBenevolence(bTime);
              myTrustHistory[temp.manufactId].addIntegrity(dTime);
              manu.myTrustHistory[temp.supplierId].addAbility(sCTime);
              manu.myTrustHistory[temp.supplierId].addBenevolence(sBTime);
              manu.myTrustHistory[temp.supplierId].addIntegrity(sDTime);
              temp.triggered = true;
          }
      }
      //return this;
  }

  public void choiceM(int quantity){
      manuQuotePrice = new int[mNum];
      manuQuoteTime = new int[mNum];
      manuChoice = new double[mNum];
      max_index = 0;
      for (int i = 0; i < mNum; i++){
          manuQuotePrice[i] = ((Manufacturer)manuList.atOffset(i)).queryPrice(quantity);
          manuQuoteTime[i] = ((Manufacturer)manuList.atOffset(i)).queryTime(quantity);
          manuChoice[i] = (manuQuotePrice[i] + manuQuoteTime[i])/2;
      }
      for (int i = 1; i < mNum; i++){
        if (manuChoice[i] > manuChoice[max_index])
            max_index = i;
      }
  }

  public void assignOrderM(int orderId){
      Order temp = (Order)orderList.atOffset(orderId);
      temp.setManuId(this.max_index);
      int sId = ((Manufacturer)manuList.atOffset(this.max_index)).max_index;
      temp.setSupplierId(sId);
      temp.setOrderPrice(this.manuQuotePrice[this.max_index]);
      temp.setContractTime(this.max_index);
  }

  public void exec_OrderM(){
      for (int i = 0 ; i < orderList.getCount(); i++){
          Order temp = (Order)orderList.atOffset(i);
          Manufacturer manu = (Manufacturer) manuList.atOffset(temp.manufactId);
          while(temp.triggerTime == Globals.env.getCurrentTime()&& temp.triggered == false){
              this.choice(temp.orderQuantity);
              this.assignOrder(i);
              int sCTime = manu.getCheckTime(temp.supplierId);
              int sDTime = manu.getDeliveryTime(temp.supplierId);
              int sBTime = manu.getBargainTime(temp.supplierId);
              int cTime = this.getCheckTime(temp.manufactId) + sCTime;
              int dTime = this.getDeliveryTime(temp.manufactId) + sDTime;
              int bTime = this.getBargainTime(temp.manufactId) + sBTime;
              myTrustHistory[temp.manufactId].addAbility(cTime);
              myTrustHistory[temp.manufactId].addBenevolence(bTime);
              myTrustHistory[temp.manufactId].addIntegrity(dTime);
              manu.myTrustHistory[temp.supplierId].addAbility(sCTime);
              manu.myTrustHistory[temp.supplierId].addBenevolence(sBTime);
              manu.myTrustHistory[temp.supplierId].addIntegrity(sDTime);
              temp.triggered = true;
          }
      }
  }

  public void assignOrderH(int orderId){
      Order temp = (Order)orderList.atOffset(orderId);
      temp.setManuId(customerId);
      temp.setSupplierId(customerId);
      temp.setContractTime(((Manufacturer)manuList.atOffset(customerId)).queryTime(temp.orderQuantity));
      temp.setOrderPrice(((Manufacturer)manuList.atOffset(customerId)).queryPrice(temp.orderQuantity));
  }

  public void exec_OrderH(){
    for (int i = 0; i < orderList.getCount(); i++){
          Order temp = (Order) orderList.atOffset(i);
          Manufacturer manu = (Manufacturer) manuList.atOffset(customerId);
          while (temp.triggerTime == Globals.env.getCurrentTime()&& temp.triggered == false){
              this.assignOrderH(i);
              int sCTime = manu.getCheckTime(temp.supplierId);
              int sDTime = manu.getDeliveryTime(temp.supplierId);
              int sBTime = manu.getBargainTime(temp.supplierId);
              int cTime = this.getCheckTime(temp.manufactId) + sCTime;
              int dTime = this.getDeliveryTime(temp.manufactId) + sDTime;
              int bTime = this.getBargainTime(temp.manufactId) + sBTime;
              myTrustHistory[temp.manufactId].addAbility(cTime);
              myTrustHistory[temp.manufactId].addBenevolence(bTime);
              myTrustHistory[temp.manufactId].addIntegrity(dTime);
              manu.myTrustHistory[temp.supplierId].addAbility(sCTime);
              manu.myTrustHistory[temp.supplierId].addBenevolence(sBTime);
              manu.myTrustHistory[temp.supplierId].addIntegrity(sDTime);
              temp.triggered = true;
          }
      }
  }
  // choice the best manufacturer end;

  public int getDeliveryTime(int mId){
      int deliveryTime;
      double deliverTimeMean = myTrustHistory[mId].getIMean();
      double deliverTimeVar = myTrustHistory[mId].getIVar();
      deliveryTime = (int)normalAbilityDist.getSampleWithMean$withVariance(deliverTimeMean, deliverTimeVar);
      if (deliveryTime <= 1)
          return 1;
      else return deliveryTime;
  }

  public int getCheckTime(int mId){
      int checkTime;
      double checkTimeMean = myTrustHistory[mId].getAMean();
      double checkTimeVar = myTrustHistory[mId].getAVar();
      checkTime = (int)normalAbilityDist.getSampleWithMean$withVariance(checkTimeMean, checkTimeVar);
      if (checkTime <=1)
          return 1;
      else return checkTime;
  }

  public int getBargainTime(int mId){
      int bargainTime;
      double bargainTimeMean = myTrustHistory[mId].getBMean();
      double bargainTimeVar = myTrustHistory[mId].getBVar();
      bargainTime = (int)normalBenevolenceDist.getSampleWithMean$withVariance(bargainTimeMean, bargainTimeVar);
      if (bargainTime <= 1)
          return 1;
      else return bargainTime;
  }
}