import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.collections.List;
import swarm.collections.ListImpl;
import swarm.activity.Activity;
import swarm.activity.ActionGroupImpl;
import swarm.activity.ScheduleImpl;
import swarm.objectbase.EmptyProbeMap;
import swarm.objectbase.EmptyProbeMapImpl;
import swarm.random.MT19937gen;
import swarm.random.MT19937genImpl;
import swarm.random.UniformIntegerDist;
import swarm.random.UniformIntegerDistImpl;
import swarm.Selector;

public class ScnModelSwarm extends SwarmImpl {

  public int numberOfCustomers = 2;
  public int numberOfManufacturers = 2;
  public int numberOfSuppliers = 2;
  public int orderTimeInterval = 10; // one week.
  public int orderTimeVariation = 5; // 3 days.
  public int orderQuantity = 2;
  public int orderQuantityVariation = 3;
  public int simulateTimePeriod = 2000;// 6 months.
  public double interestRate = 0.9;

  public int m_max_price = 9000;
  public int m_min_price = 7000;
  public int max_price = 100000;
  public int min_price = 130000;
  public int m_max_dtime = 2;
  public int m_min_dtime = 1;
  public int max_dtime = 3;
  public int min_dtime = 2;
  public int rebuildTime = 30;

  public ListImpl[] orderList;
  public ListImpl[] orderListH;
  public ListImpl[] orderListM;

  public ListImpl supplierList;
  public ListImpl supplierListH;
  public ListImpl supplierListM;

  public ListImpl manufacturerList;
  public ListImpl manufacturerListH;
  public ListImpl manufacturerListM;

  public ListImpl customerList;
  public ListImpl customerListH;
  public ListImpl customerListM;

  public MT19937gen generator;
  public int randSeed;
  public UniformIntegerDist UniTrigDist;
  public UniformIntegerDist UniQuanDist;

  public ScheduleImpl modelSchedule;

  public ScnModelSwarm(Zone aZone) {
      super(aZone);
      randSeed = 1234567890;
      generator = new MT19937genImpl(aZone, randSeed);
      UniTrigDist = new UniformIntegerDistImpl(aZone, generator);
      UniQuanDist = new UniformIntegerDistImpl(aZone, generator);

  }

  public Object buildObjects(){
    super.buildObjects();

    this.creatOrder(numberOfCustomers);
    this.creatSupplier(numberOfSuppliers);
    this.creatManufacturer(numberOfManufacturers);
    this.creatCustomer(numberOfCustomers);

    this.creatOrderH(numberOfCustomers);
    this.creatSupplierH(numberOfCustomers);
    this.creatManufacturerH(numberOfCustomers);
    this.creatCustomerH(numberOfCustomers);

    this.creatOrderM(numberOfCustomers);
    this.creatSupplierM(numberOfSuppliers);
    this.creatManufacturerM(numberOfManufacturers);
    this.creatCustomerM(numberOfCustomers);

    return this;
  }

  public Object buildActions(){
    super.buildActions();
    ActionGroupImpl modelActions = new ActionGroupImpl(this.getZone());
    try{
    modelActions.createActionForEach$message(customerList,SwarmUtils.getSelector("Customer", "exec_Order"));
    } catch (Exception e)
	{
	System.err.println("Exception Customer: exec_Order" + e.getMessage ());
	System.exit(1);
	}

    try{
    modelActions.createActionForEach$message(customerListH,SwarmUtils.getSelector("Customer", "exec_OrderH"));
    } catch (Exception e)
	{
	System.err.println("Exception Customer:exec_OrderH " + e.getMessage ());
	System.exit(1);
	}

    try{
    modelActions.createActionForEach$message(customerListM,SwarmUtils.getSelector("Customer","exec_OrderM"));
    } catch (Exception e)
	{
	System.err.println("Exception Customer:exec_OrderM " + e.getMessage ());
	System.exit(1);
	}

   	try
        {
        Selector sel;
        sel = new Selector(this.getClass(), "checkTime", false);
        modelActions.createActionTo$message(this, sel);
          } catch (Exception e)
	    {
            System.err.println("Exception checkTime: " + e.getMessage ());
	    System.exit(1);
	    }



    modelSchedule = new ScheduleImpl(this.getZone(),1);
    modelSchedule.at$createAction(0, modelActions);
    return this;
  }


  public Activity activateIn(Swarm swarmContext){
	super.activateIn(swarmContext);
	// Then activate the schedule in ourselves.
	modelSchedule.activateIn(this);
	// Finally, return the activity we have built.
	return getActivity();
  }

  public void creatOrder(int cNum){
    orderList = new ListImpl[cNum];
    for (int i = 0; i < cNum; i++){
        int time = 0;
        orderList[i] = new ListImpl(Globals.env.globalZone);
        int j = 0;
        while(time < simulateTimePeriod){
            int orderId = 0;
            int quantity = UniQuanDist.getIntegerWithMin$withMax(orderQuantity - orderQuantityVariation, orderQuantity + orderQuantityVariation);
            int triggerTime = time + UniTrigDist.getIntegerWithMin$withMax(orderTimeInterval - orderTimeVariation, orderTimeInterval + orderTimeVariation);
            Order aOrder = new Order(orderId++, quantity, triggerTime);
            time = triggerTime;
            orderList[i].addLast(aOrder);

            System.out.println("orderList" + i + " order" + j +" created");
            j++;
        }// end of while
    }// end of for
  }// end of creatOrder

  public void creatOrderH(int cNum){
    orderListH = new ListImpl[cNum];
    for (int i = 0; i < cNum; i++){
        int time = 0;
        orderListH[i] = new ListImpl(Globals.env.globalZone);
        int j = 0;
        while(time < simulateTimePeriod){
            int orderId = 0;
            int triggerTime;
            int quantity;
            quantity = UniQuanDist.getIntegerWithMin$withMax(orderQuantity - orderQuantityVariation, orderQuantity + orderQuantityVariation);
            triggerTime = time + UniTrigDist.getIntegerWithMin$withMax(orderTimeInterval - orderTimeVariation, orderTimeInterval + orderTimeVariation);
            Order aOrder = new Order(orderId++, quantity, triggerTime);
            time = triggerTime;
            orderListH[i].addLast(aOrder);
            System.out.println("orderListH" +i+ " order" +j+ " created");
            j++;
        }// end of while
    }// end of for
  }// end of creatOrderH.

  public void creatOrderM(int cNum){
    orderListM = new ListImpl[cNum];
    for (int i = 0; i < cNum; i++){
        int time = 0;
        orderListM[i] = new ListImpl(Globals.env.globalZone);
        int j = 0;
        while(time < simulateTimePeriod){
            int orderId = 0;
            int quantity = UniQuanDist.getIntegerWithMin$withMax(orderQuantity - orderQuantityVariation, orderQuantity + orderQuantityVariation);
            int triggerTime = time + UniTrigDist.getIntegerWithMin$withMax(orderTimeInterval - orderTimeVariation, orderTimeInterval + orderTimeVariation);
            Order aOrder = new Order(orderId++, quantity, triggerTime);
            time = triggerTime;
            orderListM[i].addLast(aOrder);
            System.out.println("orderListM" +i+ " order" +j+ " created");
            j++;
        }// end of while
    }// end of for
  }// end of creatOrderM

  public void creatSupplier(int sNum){
    supplierList = new ListImpl(Globals.env.globalZone);
    Supplier aSupplier;
    for (int i = 0; i < sNum; i++){
      aSupplier = new Supplier(Globals.env.globalZone, this, generator, i);
      supplierList.addLast(aSupplier);
      System.out.println("supplier " + i + " created");
    }
  }

  public void creatSupplierH(int cNum){
    supplierListH = new ListImpl(Globals.env.globalZone);
    Supplier aSupplier;
    for (int i = 0; i < cNum; i++){
      aSupplier = new Supplier(Globals.env.globalZone, this, generator, i);
      supplierListH.addLast(aSupplier);
      System.out.println("supplier H" + i + " created");
    }
  }

  public void creatSupplierM(int sNum){
    supplierListM = new ListImpl(Globals.env.globalZone);
    Supplier aSupplier;
    for (int i = 0; i < sNum; i++){
      aSupplier = new Supplier(Globals.env.globalZone, this, generator, i);
      supplierListM.addLast(aSupplier);
      System.out.println("supplier M" + i + " created");
    }
  }

  public void creatManufacturer(int mNum){
    manufacturerList = new ListImpl(Globals.env.globalZone);
    Manufacturer aManu;
    for (int i = 0; i < mNum; i++){
      aManu = new Manufacturer(Globals.env.globalZone, this, i,supplierList, generator);
      manufacturerList.addLast(aManu);
      System.out.println("manufacturer" + i + " created");
    }
  }

  public void creatManufacturerH(int cNum){
    manufacturerListH = new ListImpl(Globals.env.globalZone);
    Manufacturer aManu;
    for (int i = 0; i < cNum; i++){
      aManu = new Manufacturer(Globals.env.globalZone, this, i, supplierListH, generator);
      manufacturerListH.addLast(aManu);
      System.out.println("manufacturer H" + i + " created");
    }
  }

  public void creatManufacturerM(int mNum){
    manufacturerListM = new ListImpl(Globals.env.globalZone);
    Manufacturer aManu;
    for (int i = 0; i < mNum; i++){
      aManu = new Manufacturer(Globals.env.globalZone, this, i, supplierListM, generator);
      manufacturerListM.addLast(aManu);
      System.out.println("manufacturer M" + i + " created");
    }
  }

  public void creatCustomer(int cNum){
    customerList = new ListImpl(Globals.env.globalZone);
    Customer aCustomer;
    for (int i = 0; i < cNum; i++){
      aCustomer = new Customer(Globals.env.globalZone, this, orderList[i], manufacturerList, generator, i);
      customerList.addLast(aCustomer);
      System.out.println("customer" + i + " created");
    }
  }

  public void creatCustomerH(int cNum){
    customerListH = new ListImpl(Globals.env.globalZone);
    Customer aCustomer;
    for (int i = 0; i < cNum; i++){
      aCustomer = new Customer(Globals.env.globalZone, this, orderListH[i], manufacturerListH, generator, i);
      customerListH.addLast(aCustomer);
      System.out.println("customer H" + i + " created");
    }
  }

  public void creatCustomerM(int cNum){
    customerListM = new ListImpl(Globals.env.globalZone);
    Customer aCustomer;
    for(int i = 0; i < cNum; i++){
      aCustomer = new Customer(Globals.env.globalZone, this, orderListM[i], manufacturerListM, generator, i);
      customerListM.addLast(aCustomer);
      System.out.println("customer M" + i + " created");
    }
  }

   public void checkTime()
   {
	int i, t;
	int interval = 5;

	if (Globals.env.getCurrentTime() >= simulateTimePeriod)
	    {
	    System.out.println("We've reached our endTime at period " + Globals.env.getCurrentTime());
	    Globals.env.getCurrentSwarmActivity().terminate();
	    }
	else
	    {
		for (i = 0; (t = (int)Math.pow(interval, (double)i)) <= simulateTimePeriod; i++)
		    if (t == Globals.env.getCurrentTime())
			System.out.println("The time is " + t);
	    }
        return;
  }

}